'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

function getDefaultExportFromCjs (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, basedir, module) {
	return module = {
		path: basedir,
		exports: {},
		require: function (path, base) {
			return commonjsRequire(path, (base === undefined || base === null) ? module.path : base);
		}
	}, fn(module, module.exports), module.exports;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

var objectWithoutPropertiesLoose = createCommonjsModule(function (module) {
function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

module.exports = _objectWithoutPropertiesLoose;
module.exports["default"] = module.exports, module.exports.__esModule = true;
});

var _objectWithoutPropertiesLoose = /*@__PURE__*/getDefaultExportFromCjs(objectWithoutPropertiesLoose);

var _extends_1 = createCommonjsModule(function (module) {
function _extends() {
  module.exports = _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  module.exports["default"] = module.exports, module.exports.__esModule = true;
  return _extends.apply(this, arguments);
}

module.exports = _extends;
module.exports["default"] = module.exports, module.exports.__esModule = true;
});

var _extends = /*@__PURE__*/getDefaultExportFromCjs(_extends_1);

var _excluded$2 = ["prefix", "viewConfig", "mediaConfig"];
var mergeConfig = function mergeConfig(props, config) {
  var prefix = config.prefix,
      viewConfig = config.viewConfig,
      mediaConfig = config.mediaConfig,
      initConfig = _objectWithoutPropertiesLoose(config, _excluded$2);

  var configs = {
    viewConfig: viewConfig,
    mediaConfig: mediaConfig,
    initConfig: initConfig
  };
  return Object.assign.apply(Object, [{}, Object.fromEntries(props.filter(function (p) {
    return prefix.some(function (key) {
      return key === p[0];
    });
  }))].concat(Object.entries(configs).map(function (_ref) {
    var key = _ref[0],
        conf = _ref[1];
    return _extends({}, (props.find(function (p) {
      return p[0] === key;
    }) || [0, {}])[1], conf);
  })));
};

var is = function is(a, b) {
  for (var _len = arguments.length, other = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    other[_key - 2] = arguments[_key];
  }

  if (other.length > 0) return is(a, b) && is.apply(void 0, [b].concat(other));
  if (typeof a !== typeof b) return false;
  if (is.str(a) || is.num(a)) return a === b;

  for (var i in a) {
    if (!(i in b)) return false;
  }

  for (var _i in b) {
    if (a[_i] !== b[_i]) return false;
  }

  return true;
};

is.arr = Array.isArray;

is.fls = function (a) {
  return is.und(a) || is.nul(a) || a === false || a === '';
};

is.nul = function (a) {
  return a === null;
};

is.und = function (a) {
  return a === void 0;
};

is.num = function (a) {
  return typeof a === 'number';
};

is.str = function (a) {
  return typeof a === 'string';
};

is.bol = function (a) {
  return typeof a === 'boolean';
};

is.fun = function (a) {
  return typeof a === 'function';
};

is.obj = function (a) {
  return Object.prototype.toString.call(a) === '[object Object]';
};

is.url = function (a) {
  return a instanceof URL;
};

is.set = function (a) {
  return a instanceof Set;
};

is.map = function (a) {
  return a instanceof Map;
};

is.big = function (a) {
  return is.str(a) && a === a.toUpperCase();
};

is.len = function (l, a) {
  return (is.arr(a) || is.str(a)) && a.length === l;
};

var defaultMedia = {
  media: '',
  matches: false,
  onchange: function onchange() {},
  addListener: function addListener() {},
  removeListener: function removeListener() {},
  addEventListener: function addEventListener() {},
  removeEventListener: function removeEventListener() {},
  dispatchEvent: function dispatchEvent(_) {
    return true;
  }
};
var defaultSize = {
  xs: 1,
  sm: 576,
  md: 768,
  lg: 992,
  xl: 1200
};
var defaultConfig = {
  mediaConfig: {
    size: defaultSize,
    width: 0,
    widthRef: null,
    mediaType: null
  },
  viewConfig: {
    once: false,
    onView: null,
    timeout: 0,
    defaultView: true
  },
  prefix: ['none', 'init', 'onView', 'mediaConfig', 'viewConfig']
};
function convertNumToPix(value, config) {
  if (config === void 0) {
    config = defaultConfig;
  }

  var target = config.widthRef;
  if (target && 'current' in target) target = target.current;
  var width = target ? target.clientWidth : config.width || 0; // todo window.innerWidth

  if (is.num(value) && Math.pow(value, 2) < 1) return value < 0 ? width * (Number(value) + 1) << 0 : width * Number(value) << 0;

  if (value instanceof Array && value.every(function (v) {
    return is.num(v);
  })) {
    var border = value.filter(function (v) {
      return v > 0;
    }).map(function (v) {
      return v < 1 ? width * Number(v) : v;
    });
    var margin = width - (border.length ? border.reduce(function (a, b) {
      return a + b;
    }) : 0);
    return value.map(function (v) {
      return v === 0 ? 0 : v < 0 ? v > -1 ? ~~(margin * Number(-v)) : v : v < 1 ? ~~(width * Number(v)) : v;
    });
  }

  return value;
}
function convertObjToStr(query, config) {
  if (config === void 0) {
    config = defaultConfig;
  }

  if (is.str(query)) return query;
  return [config.mediaType || ''].concat(Object.entries(query).map(function (_ref) {
    var key = _ref[0],
        val = _ref[1];
    return [key.replace(/[A-Z]/g, function (s) {
      return "-" + s.toLowerCase();
    }).toLowerCase(), val];
  }).map(function (_ref2) {
    var key = _ref2[0],
        val = _ref2[1];
    return is.bol(val) ? "" + (val ? '' : 'not ') + key : "(" + key + ": " + (is.num(val) && /[height|width]$/.test(key + "") ? convertNumToPix(val, config) + "px" : val) + ")";
  })).filter(Boolean).join(' and ');
}

function convertPrefixToList(props, config) {
  if (config === void 0) {
    config = defaultConfig;
  }

  var _config = config,
      _config$size = _config.size,
      size = _config$size === void 0 ? defaultSize : _config$size;
  var keys = Object.keys(size);

  var toN = function toN(key) {
    return keys.find(function (k) {
      return k === key;
    }) ? size[key] : 0;
  };

  var toS = function toS(key, i, grid) {
    return "(min-width:" + toN(key) + "px)" + (i + 1 - grid.length < 0 ? " and (max-width:" + (toN(grid[i + 1][0]) - 1) + "px)" : '');
  }; //const rmNone = (key:string):string=>key.replace(/(-none|none)/gi, '')  // ??


  var grid = keys.map(function (s) {
    return props.find(function (p) {
      return p[0] === s;
    }) || null;
  }).filter(function (m) {
    return m !== null;
  });
  var xsGr = grid.length ? grid.find(function (g) {
    return g[0] === "xs";
  }) ? [] : [["xs", grid[0][1]]] : [];
  var gtos = [].concat(xsGr, grid).map(function (g, i) {
    return [toS(g[0], i, grid), g[1]];
  });
  var noGr = grid.length ? props.filter(function (p) {
    return !keys.find(function (s) {
      return s === p[0];
    });
  }) : props;
  return [].concat(noGr, gtos);
}
function convertPropsToList(props, config) {
  if (config === void 0) {
    config = defaultConfig;
  }

  var prefix = props.map(function (_ref) {
    var key = _ref[0],
        val = _ref[1];
    return [convertObjToStr(key, config), convertNumToPix(val, config)];
  });
  return convertPrefixToList(prefix, config);
}
function convertFuncionToList(props) {
  var list = props.map(function (p) {
    return p instanceof Array ? p : Object.entries(p);
  });
  return list[0].map(function (p) {
    return p[0];
  }).map(function (key) {
    return [key, list.map(function (p) {
      return p.find(function (v) {
        return v[0] === key;
      })[1];
    })];
  });
}

var createGrid = function createGrid(effect) {
  return function (initProps, targetRef, initConfig) {
    var _targetRef;

    if (targetRef === void 0) {
      targetRef = [];
    }

    if (initConfig === void 0) {
      initConfig = defaultConfig;
    }

    if (is.fun(initProps)) initProps = initProps();
    if (is.len(0, targetRef)) targetRef = [];
    var props = is.arr(initProps) ? initProps : Object.entries(initProps);

    var _useState = React.useState(function () {
      return mergeConfig(props, initConfig);
    }),
        config = _useState[0]; // ********** ➊ grid: output value that match your media query ********** //


    var _useState2 = React.useState(convertPropsToList(props, config)),
        list = _useState2[0],
        setList = _useState2[1];

    var _useState3 = React.useState(config.defaultView),
        view = _useState3[0],
        setView = _useState3[1];

    var _useState4 = React.useState(function () {
      var prop = list.find(function (l) {
        return l[0] === "init";
      }) || list[0];
      return (prop == null ? void 0 : prop.length) > 1 ? prop[1] : null; // TODO
    }),
        grid = _useState4[0],
        setGrid = _useState4[1];

    var noneRef = React.useRef();
    var gridRef = React.useRef(initProps);
    var viewRef = React.useRef(Array((_targetRef = targetRef) == null ? void 0 : _targetRef.length).fill(view)); // ********** ➋ set: Functions to change media conditions later ********** //

    var set = React.useCallback(function (initState) {
      if (is.fun(initState)) initState = initState(gridRef.current);
      gridRef.current = initState;
      var state = Array.isArray(initState) ? initState : Object.entries(initState);
      setList(convertPropsToList(state, config));
    }, [config]); // ********** 📺 ➌ effect: for useMedia 📺 ********** //

    effect(function () {
      var mounted = true;
      var prefix = config.prefix;
      var mediaSet = list.map(function (_ref) {
        var query = _ref[0],
            value = _ref[1];
        if (query === "none") noneRef.current = value;
        if (prefix != null && prefix.some(function (q) {
          return q === query;
        })) return {
          media: null,
          fn: null
        };
        var media = is.und(window) ? defaultMedia : window.matchMedia(query);

        var fn = function fn() {
          if (mounted && media.matches) setGrid(value);
        };

        media.addListener(fn);
        if (mounted && media.matches) setGrid(value);
        return {
          media: media,
          fn: fn
        };
      });
      return function () {
        mounted = false;
        mediaSet.map(function (_ref2) {
          var media = _ref2.media,
              fn = _ref2.fn;
          return fn && (media == null ? void 0 : media.removeListener(fn));
        });
      };
    }, [list, config]); // ********** 👀 ➍ effect : for useView 👀 ********** //

    effect(function () {
      var _targetRef2;

      var mounted = true;
      var _config$timeout = config.timeout,
          timeout = _config$timeout === void 0 ? 0 : _config$timeout,
          _config$once = config.once,
          once = _config$once === void 0 ? false : _config$once,
          _config$onView = config.onView,
          onView = _config$onView === void 0 ? null : _config$onView;
      var observers = (_targetRef2 = targetRef) == null ? void 0 : _targetRef2.map(function (ref, i) {
        var _config$root;

        var el = (ref == null ? void 0 : ref.current) || ref;
        if (!el) return {
          observer: null,
          el: el
        };
        var observer = new IntersectionObserver(function (entries) {
          var entry = entries[entries.length - 1];
          if (!mounted) return;
          setTimeout(function () {
            viewRef.current[i] = entry.isIntersecting;
            if (onView) onView(entry.isIntersecting); // TODO input entry

            if (noneRef.current) setView(viewRef.current.some(Boolean));
          }, timeout);
          if (entry.isIntersecting && once) observer.unobserve(el);
        }, {
          root: (config == null ? void 0 : (_config$root = config.root) == null ? void 0 : _config$root.current) || null
        });
        observer.observe(el);
        return {
          observer: observer,
          el: el
        };
      });
      return function () {
        mounted = false;
        if (once) observers.map(function (_ref3) {
          var observer = _ref3.observer,
              el = _ref3.el;
          return el && (observer == null ? void 0 : observer.unobserve(el));
        });
      };
    }, [config]);
    return [view || !noneRef.current ? grid : noneRef.current, set];
  };
};

var createGrids = function createGrids(useGrid) {
  return function (length, props, refs, cofig) {
    if (refs === void 0) {
      refs = [];
    }

    if (cofig === void 0) {
      cofig = defaultConfig;
    }

    if (is.fun(props)) props = [].concat(Array(length)).map(function (_, i) {
      return props(i);
    });

    var _useGrid = useGrid(convertFuncionToList(props), refs, cofig),
        grids = _useGrid[0],
        set = _useGrid[1];

    var setGrid = function setGrid(state) {
      if (is.fun(state)) state = [].concat(Array(length)).map(function (_, i) {
        return state(i);
      });
      set(convertFuncionToList(state));
    };

    return [grids, setGrid];
  };
};

// ************************************************** //
var createMedia = function createMedia(effect) {
  return function (rawQuery, defaultState) {
    if (defaultState === void 0) {
      defaultState = false;
    }

    var query = convertObjToStr(rawQuery);

    var _useState = React.useState(defaultState),
        state = _useState[0],
        set = _useState[1];

    effect(function () {
      var mounted = true;
      var media = is.und(window) ? defaultMedia : window.matchMedia(query);

      var onChange = function onChange() {
        return mounted && set(Boolean(media.matches));
      };

      media.addListener(onChange);
      set(media.matches);
      return function () {
        return void (mounted = false, media.removeListener(onChange));
      };
    }, [query]);
    return state;
  };
};

var _excluded$1 = ["defaultView", "once", "timeout"];
var createView = function createView(effect) {
  return function (target, onView, viewConfig) {
    if (viewConfig === void 0) {
      viewConfig = defaultConfig.viewConfig;
    }

    var _viewConfig = viewConfig,
        defaultView = _viewConfig.defaultView,
        once = _viewConfig.once,
        timeout = _viewConfig.timeout,
        initialConfig = _objectWithoutPropertiesLoose(_viewConfig, _excluded$1);

    var _useState = React.useState(defaultView === true),
        view = _useState[0],
        set = _useState[1];

    var configRef = React.useRef(initialConfig);
    effect(function () {
      if (!is(configRef.current, initialConfig)) configRef.current = initialConfig;
    }, [initialConfig]);
    effect(function () {
      var _configRef$current, _configRef$current$ro;

      var mounted = true;
      var root = ((_configRef$current = configRef.current) == null ? void 0 : (_configRef$current$ro = _configRef$current.root) == null ? void 0 : _configRef$current$ro.current) || null;
      var el = target instanceof Element ? target : target == null ? void 0 : target.current;
      if (!el) return function () {
        return void (mounted = false);
      };
      var observer = new IntersectionObserver(function (entries) {
        var entry = entries[entries.length - 1];
        if (onView) onView(entry);
        if (!mounted) return;
        setTimeout(function () {
          return set(entry.isIntersecting);
        }, timeout);
        if (entry.isIntersecting && once) observer.unobserve(el);
      }, _extends({}, configRef.current, {
        root: root
      }));
      observer.observe(el);
      return function () {
        mounted = false;
        if (once) observer.unobserve(el);
      };
    }, [configRef.current, target]);
    return view;
  };
};

var _excluded = ["children", "config"],
    _excluded2 = ["children", "config"],
    _excluded3 = ["children", "init"],
    _excluded4 = ["children", "init"],
    _excluded5 = ["children", "onView"],
    _excluded6 = ["children", "onView"];
var useMedia = createMedia(React__default['default'].useEffect);
var useView = createView(React__default['default'].useEffect);
var useGrid = createGrid(React__default['default'].useEffect);
var useGrids = createGrids(useGrid);
var useLayoutMedia = createMedia(React__default['default'].useLayoutEffect);
var useLayoutView = createView(React__default['default'].useLayoutEffect);
var useLayoutGrid = createGrid(React__default['default'].useLayoutEffect);
var useLayoutGrids = createGrids(useLayoutGrid);
var Grid = /*#__PURE__*/React__default['default'].forwardRef(_Grid);
var Media = /*#__PURE__*/React__default['default'].forwardRef(_Media);
var View = /*#__PURE__*/React__default['default'].forwardRef(_View);
var LayoutGrid = /*#__PURE__*/React__default['default'].forwardRef(_LayoutGrid);
var LayoutMedia = /*#__PURE__*/React__default['default'].forwardRef(_LayoutMedia);
var LayoutView = /*#__PURE__*/React__default['default'].forwardRef(_LayoutView);

function _Grid(props, ref) {
  var children = props.children,
      config = props.config,
      other = _objectWithoutPropertiesLoose(props, _excluded);

  var target = React__default['default'].useRef(null);
  React__default['default'].useImperativeHandle(ref, function () {
    return target.current;
  });
  return children(useGrid(other, [target], config)[0], target);
}

function _LayoutGrid(props, ref) {
  var children = props.children,
      config = props.config,
      other = _objectWithoutPropertiesLoose(props, _excluded2);

  var target = React__default['default'].useRef(null);
  React__default['default'].useImperativeHandle(ref, function () {
    return target.current;
  });
  return children(useLayoutGrid(other, [target], config)[0], target);
}

function _Media(props, ref) {
  var children = props.children,
      init = props.init,
      other = _objectWithoutPropertiesLoose(props, _excluded3);

  var media = useMedia(other, init);
  React__default['default'].useImperativeHandle(ref, function () {
    return media;
  });
  return children(media);
}

function _LayoutMedia(props, ref) {
  var children = props.children,
      init = props.init,
      other = _objectWithoutPropertiesLoose(props, _excluded4);

  var media = useLayoutMedia(other, init);
  React__default['default'].useImperativeHandle(ref, function () {
    return media;
  });
  return children(media);
}

function _View(props, ref) {
  var children = props.children,
      onView = props.onView,
      config = _objectWithoutPropertiesLoose(props, _excluded5);

  var target = React__default['default'].useRef(null);
  React__default['default'].useImperativeHandle(ref, function () {
    return target.current;
  });
  return children(useView(target, onView, config), target);
}

function _LayoutView(props, ref) {
  var children = props.children,
      onView = props.onView,
      config = _objectWithoutPropertiesLoose(props, _excluded6);

  var target = React__default['default'].useRef(null);
  React__default['default'].useImperativeHandle(ref, function () {
    return target.current;
  });
  return children(useLayoutView(target, onView, config), target);
}

exports.Grid = Grid;
exports.LayoutGrid = LayoutGrid;
exports.LayoutMedia = LayoutMedia;
exports.LayoutView = LayoutView;
exports.Media = Media;
exports.View = View;
exports.convertFuncionToList = convertFuncionToList;
exports.convertNumToPix = convertNumToPix;
exports.convertObjToStr = convertObjToStr;
exports.convertPrefixToList = convertPrefixToList;
exports.convertPropsToList = convertPropsToList;
exports.defaultConfig = defaultConfig;
exports.defaultMedia = defaultMedia;
exports.defaultSize = defaultSize;
exports.is = is;
exports.mergeConfig = mergeConfig;
exports.useGrid = useGrid;
exports.useGrids = useGrids;
exports.useLayoutGrid = useLayoutGrid;
exports.useLayoutGrids = useLayoutGrids;
exports.useLayoutMedia = useLayoutMedia;
exports.useLayoutView = useLayoutView;
exports.useMedia = useMedia;
exports.useView = useView;
